@@ -11,8 +11,8 @@ WHITELIST = {
     "README.txt",
     "__init__.py",
     "vendor_any.txt",
-    "vendor_v3_5.txt",
     "vendor_v3_6.txt",
+    "vendor_v3_7.txt",
     "pip.LICENSE",
 }
 
@@ -53,11 +53,11 @@ def find_vendored_libs(vendor_dir, whitelist, whitelist_dirs):
     vendored_libs = []
     paths = []
     for item in vendor_dir.iterdir():
-        if item.is_dir() and not item in whitelist_dirs:
+        if item.is_dir() and item not in whitelist_dirs:
             vendored_libs.append(item.name)
         elif item.is_file() and item.name not in whitelist:
             vendored_libs.append(item.stem)  # without extension
-        else:  # not a dir or a file not in the whilelist
+        else:  # not a dir or a file not in the whitelist
             continue
         paths.append(item)
     return vendored_libs, paths
